<?php
/**
 * @brief		Uninstall callback
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	(DP41) Referrals System
 * @since		07 Mar 2017
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\rsystem\extensions\core\Uninstall;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Uninstall callback
 */
class _rSystemUninstall
{
	/**
	 * Code to execute before the application has been uninstalled
	 *
	 * @param	string	$application	Application directory
	 * @return	array
	 */
	public function preUninstall( $application )
	{
	}

	/**
	 * Code to execute after the application has been uninstalled
	 *
	 * @param	string	$application	Application directory
	 * @return	array
	 */
	public function postUninstall( $application )
	{
	    $fields = array( 'dp3_rs_menable', 'dp3_rs_referred_by', 'dp3_rs_banned', 'dp3_rs_padded', 'dp3_rs_incr', 'dp3_rs_hashcode' );
        
        /* Go through and delete */                      
        foreach( $fields as $field )
        {
            if( \IPS\Db::i()->checkForColumn( 'core_members', $field ) )
            {
                \IPS\Db::i()->dropColumn( 'core_members', $field );    
            }            
        }
        
        /* Delete table */
        if ( \IPS\Db::i()->checkForTable( 'rsystem_referrals' ) )
        {
            \IPS\Db::i()->dropTable( 'rsystem_referrals' );
        }
	}

	/**
	 * Code to execute when other applications are uninstalled
	 *
	 * @param	string	$application	Application directory
	 * @return	void
	 * @deprecated	This is here for backwards-compatibility - all new code should go in onOtherUninstall
	 */
	public function onOtherAppUninstall( $application )
	{
		return $this->onOtherUninstall( $application );
	}

	/**
	 * Code to execute when other applications or plugins are uninstalled
	 *
	 * @param	string	$application	Application directory
	 * @param	int		$plugin			Plugin ID
	 * @return	void
	 */
	public function onOtherUninstall( $application=NULL, $plugin=NULL )
	{
	}
}