//<?php

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

class rsystem_hook_registerStuff extends _HOOK_CLASS_
{


	/**
	 * Build Registration Form
	 *
	 * @return	\IPS\Helpers\Form
	 */
	static public function buildRegistrationForm()
	{
		try
		{
			try
			{
				$form = call_user_func_array( 'parent::buildRegistrationForm', func_get_args() );
			        
				/* System is enabled? */
				if( ! \IPS\Settings::i()->dp41_rsystem_enable )
				{                 
					return $form;
				}
		        
				/* Check the same IP is disabled to register */
				if( ! \IPS\rsystem\Settings::checkActIp() )
				{
					return $form;
				} 
		
				/* Invite mode only? */
				if ( ( \IPS\Settings::i()->dp41_rsystem_invite_type == 'invite' ) && ( ! isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] ) || ( \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] == '-' ) || ! \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] ) )
				{
					\IPS\Output::i()->error( 'rsystem_invite_must_be_invited_error', 'RSYSTEM_026', 403, '' );
				}
		        
		        if( \IPS\Settings::i()->dp41_rsystem_enable && ! \IPS\Request::i()->form_submitted )
		        {
					if( \IPS\Settings::i()->dp41_rsystem_invite_type == 'invite' )
					{
						/* Get cookie */
		                if( isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] ) )
		                {
		                    $reff_cookie = \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'];
		                }
		                else
		                {
		                    $reff_cookie = '-';
		                }
						
						/* Do we have refferal invite key? */
						if( ( $reff_cookie == '-' ) || ! $reff_cookie )	
						{
							#Remove all cookies
		                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
		                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
		
		                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_no_invite_ref_hash' );
		                    return $form;	
						}
						
						/* Get invite info from SQL */
		        		try
		                {
		                    $data = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( 'i_secure_key = ?', $reff_cookie ) )->first(); 
		        		}
		                catch ( \UnderflowException $e )
		                {}
						
						/* No invite? */
						if( ! $data['i_id'] )
						{
							#Remove all cookies
		                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
		                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
		
		                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_invite_no_such_key' );
		                    return $form;				
						}
						
						/* Invite was expired? */
						if( \IPS\Settings::i()->dp41_rsystem_invites_expire_time )
						{
							if( ( $data['i_time'] + ( \IPS\Settings::i()->dp41_rsystem_invites_expire_time * ( 60 * 60 * 24 ) ) ) < time() )
							{
		    					#Remove all cookies
		                        \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
		                        \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
		    
		                        $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_invite_expired' );
		                        return $form;					
							}
						}
						
						/* Used */
						if( $data['i_status'] != 'sent' )
						{
							#Remove all cookies
		                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
		                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
		
		                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_invite_used_already' );
		                    return $form;					
						}
						
						/* Check key if it's correct */
						if( $reff_cookie != $data['i_secure_key'] )
						{
							#Remove all cookies
		                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
		                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
		
		                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_error_invite_wrong_key' );
		                    return $form;				
						}									
					}	
		        }
		        
		        /* Small init */
		        $disable = false;		
				 
				/* Cookie setting? */
				if( isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_id'] ) )
		        {
		            $refID = \IPS\Request::i()->cookie['dp3_rs_invite_reff_id'];
				}
		        else
		        {
		            $refID = '-';
		        }
		        
		        if( isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] ) )
		        {
		            $refHash = \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'];
				}
		        else
		        {
		            $refHash = '-';
		        }
		
				if( intval( $refID ) )
				{
					$member = \IPS\Member::load( $refID );
				}
				elseif( $refHash && ( $refHash != '-' ) )
				{
		            try
		            {
		                $inviterId = \IPS\Db::i()->select( 'i_inviter_id', 'rsystem_referrals', array( 'i_secure_key = ?', $refHash ) )->first(); 
		            }
		            catch ( \UnderflowException $e )
		            {}
		
					$member = \IPS\Member::load( $inviterId );			
				}
				
				if( isset( $member ) && $member->member_id )
				{
					if( $member instanceof \IPS\Member )
		            {
		                $value = $member->name;
					
					    /* Disable */
					    $disable = true;
		            }
				}
				elseif( isset( \IPS\Request::i()->referral_name ) && mb_strlen( \IPS\Request::i()->referral_name ) )
				{
					$value = \IPS\Request::i()->referral_name;
				}
		        
				if( $refHash && ( $refHash != '-' ) )
				{
					/* Disable */
		            if( \IPS\Settings::i()->dp41_rsystem_invite_type != 'free' )
		            {
		                $disable = true;
		            }	
				}
		        
				/* Process data */
			    $form->addSeparator();
		        
		        $form->add( new \IPS\Helpers\Form\Text( 'referral_name', ( isset( $value ) ? $value : \IPS\Request::i()->referral_name ), ( \IPS\Settings::i()->dp41_rsystem_invite_type == 'force' ? TRUE : FALSE ), array( 'disabled' => $disable ), function( $val )
					{
		    			/* Do we have cookie with the hash? */
		                if( isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] ) )
		                {
		                    $reffKey = \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'];
		                }
		                else
		                {
		                    $reffKey = '-';
		                }
		
		    			if( $reffKey && ( $reffKey != '-' ) )
		    			{
		            		try
		                    {
		                        $_data = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( 'i_secure_key = ?', $reffKey ) )->first(); 
		            		}
		                    catch ( \UnderflowException $e )
		                    {}
		
		    				if( isset( $_data['i_inviter_id'] ) )
		    				{
		    					$inviter = \IPS\Member::load( $_data['i_inviter_id'] );
		    					
		    					$referralName = $inviter->name;					
		    				}	
		    			}
		    			else
		    			{
		    				$referralName = $val;	
		    			}
		                
		    			/* Check entered name */   			
		    			if( isset( $referralName ) )
		                {
		                    $checkKey = \IPS\rsystem\Settings::checkEnteredName( $referralName );
		                    
		        			/* Do we have error? */
		        			if( ! is_numeric( $checkKey ) )
		        			{
		        				throw new \DomainException( $checkKey );			
		        			}
		                }
					}, NULL, NULL, 'referral_name' ) );
		
		        if( \IPS\Settings::i()->dp41_rsystem_invite_type == 'invite' )
		        {
		            \IPS\Lang::saveCustom( 'rsystem', 'referral_name_desc', \IPS\Member::loggedIn()->language()->get( 'referral_name_invite' ) );     
		        }
		        elseif( \IPS\Settings::i()->dp41_rsystem_invite_type == 'force' )
		        {
		            \IPS\Lang::saveCustom( 'rsystem', 'referral_name_desc', \IPS\Member::loggedIn()->language()->get( 'referral_name_force' ) );     
		        }
		        else
		        {
		            \IPS\Lang::saveCustom( 'rsystem', 'referral_name_desc', \IPS\Member::loggedIn()->language()->get( 'referral_name_free' ) );      
		        }
		        
		        return $form;
			}
			catch ( \RuntimeException $e )
			{
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}

	/**
	 * Create Member
	 *
	 * @param array $values       Values from form
	 * @param array $profileFields    Profile field values from registration
	 * @return  \IPS\Member
	 */
	public static function _createMember( $values, $profileFields )
	{
		try
		{
			try
			{
				$member = call_user_func_array( 'parent::_createMember', func_get_args() );
		        
		        if( \IPS\Settings::i()->dp41_rsystem_enable )
		        {
					/* Do we have cookie with the hash? */
		            if( isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] ) )
		            {
		                $reffKey = \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'];
		            }
		            else
		            {
		                $reffKey = '-';
		            }
		
					if( $reffKey && ( $reffKey != '-' ) )
					{
		        		try
		                {
		                    $_data = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( 'i_secure_key = ?', $reffKey ) )->first(); 
		        		}
		                catch ( \UnderflowException $e )
		                {}
		
						if( isset( $_data['i_inviter_id'] ) )
						{
							$inviter      = \IPS\Member::load( $_data['i_inviter_id'] );
							$referralName = $inviter->name;					
						}
		                else
		                {
		                    $reffKey      = '-';
		                }	
					}
		            
		            if( isset( $values['referral_name'] ) && mb_strlen( $values['referral_name'] ) )
					{
						$inviter = \IPS\Member::load( $values['referral_name'], 'name' );
						$referralName = $inviter->name;		
					}
		            
		            if( ! $referralName )
		            {
		                return $member;
		            }
		
		            if( isset( $referralName ) && mb_strlen( $referralName ) )
		            {
		                try
		                {
		                    $referredBy = \IPS\Member::load( $referralName, 'name' );
		                     
		            		/* Add referred by log */        		
		            		if( isset( $referredBy->member_id ) && $referredBy->member_id )
		            		{
		            		    $member->dp3_rs_referred_by = $referredBy->member_id;
		                        
		            		    if( ( ( $reffKey == '-' ) || ! $reffKey ) && \IPS\rsystem\Settings::checkActIp() )	
		            		    {
		                            $newKey = (string) md5( $_SERVER['REMOTE_ADDR'] . ')(&^$#!%^&E$^*)' . uniqid( microtime(), true ) );
		                            \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', $newKey, \IPS\DateTime::create()->add( new \DateInterval( 'P1Y') ), FALSE );
		                           
		                            $log                 = new \IPS\rsystem\Log;
		                            $log->time           = time();
		                            $log->inviter_id     = $referredBy->member_id;
		                            $log->friend_mail    = $member->email;
		                            $log->secure_key     = $newKey; 
		                            $log->status         = 'sent';
		                            $log->invited_ip     = \IPS\Member::loggedIn()->ip_address;
		                            $log->user_pending   = $member->member_id;
		                            $log->save();			          
		            			}
		            			else
		            			{
		                            \IPS\Db::i()->update( 'rsystem_referrals', array( 'i_user_pending' => $member->member_id ), array( "i_secure_key = ?", (string) $reffKey ) );
		            			}
		            		}
		                }
		                catch ( \OutOfRangeException $e )
		                {
		                    return $member;
		                }
		            }
		            else
		            {
		                $member->dp3_rs_referred_by = 0;    
		            }
		            
		            $member->dp3_rs_padded = \IPS\Settings::i()->dp41_rsystem_minimum_posts ? 0 : 1;
		        }
		        
		        return $member;
			}
			catch ( \RuntimeException $e )
			{
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \RuntimeException $e )
		{
			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return call_user_func_array( 'parent::' . __FUNCTION__, func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
}
