<?php


namespace IPS\rsystem\modules\admin\settings;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * settings
 */
class _settings extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'settings_manage' );
		parent::execute();
        
        \IPS\Output::i()->output .= \IPS\Application::load( 'rsystem' )->cpr();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$form = new \IPS\Helpers\Form;
	    
        $form->addTab( 'rsystem_main_settings' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'dp41_rsystem_enable', \IPS\Settings::i()->dp41_rsystem_enable, TRUE ) );

        $groups = array();
        foreach ( \IPS\Member\Group::groups() as $k => $v )
        {
        	$groups[ $v->g_id ] = $v->name;
        }
        
        $form->add( new \IPS\Helpers\Form\Select( 'dp41_rsystem_groups', \IPS\Settings::i()->dp41_rsystem_groups != 'all' ? explode( ',', \IPS\Settings::i()->dp41_rsystem_groups ) : 'all', FALSE, array( 'options' => $groups, 'parse' => 'normal', 'multiple' => true, 'unlimited' => 'all', 'unlimitedLang' => 'all_groups' ), NULL, NULL, NULL, 'dp41_rsystem_groups' ) );
        $form->add( new \IPS\Helpers\Form\YesNo( 'dp41_rsystem_enable_mask', \IPS\Settings::i()->dp41_rsystem_enable_mask, TRUE ) );
        $form->add( new \IPS\Helpers\Form\Number( 'dp41_rsystem_minimum_posts', \IPS\Settings::i()->dp41_rsystem_minimum_posts, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'rsystem_disabled_option' ) ) );
        $form->add( new \IPS\Helpers\Form\YesNo( 'dp41_rsystem_show_referred_info_in_profile', \IPS\Settings::i()->dp41_rsystem_show_referred_info_in_profile, TRUE ) );
        
        $redirectOpts[ 'index' ]    = \IPS\Member::loggedIn()->language()->addToStack( 'dp41_rsystem_redirect_to_index' );
        $redirectOpts[ 'register' ] = \IPS\Member::loggedIn()->language()->addToStack( 'dp41_rsystem_redirect_to_register_form' );
        $redirectOpts[ 'custom' ] = \IPS\Member::loggedIn()->language()->addToStack( 'dp41_rsystem_redirect_to_custom_url' );
        
        $form->add( new \IPS\Helpers\Form\Select( 'dp41_rsystem_redirect_to', \IPS\Settings::i()->dp41_rsystem_redirect_to, TRUE, array( 'options' => $redirectOpts, 'parse' => 'normal', 'multiple' => false, 'toggles' => array( 'custom' => array( 'dp41_rsystem_redirect_to_custom_url_url' ) ) ), NULL, NULL, NULL, 'dp41_rsystem_redirect_to' ) );
        $form->add( new \IPS\Helpers\Form\Url( 'dp41_rsystem_redirect_to_custom_url_url', \IPS\Settings::i()->dp41_rsystem_redirect_to_custom_url_url, false, array(), NULL ,NULL ,NULL, 'dp41_rsystem_redirect_to_custom_url_url' ) );

        $inviteOpts[ 'free' ]    = \IPS\Member::loggedIn()->language()->addToStack( 'dp41_rsystem_invite_type_free' );
        $inviteOpts[ 'force' ] = \IPS\Member::loggedIn()->language()->addToStack( 'dp41_rsystem_invite_type_force' );
        $inviteOpts[ 'invite' ] = \IPS\Member::loggedIn()->language()->addToStack( 'dp41_rsystem_invite_type_invite' );
        $form->add( new \IPS\Helpers\Form\Select( 'dp41_rsystem_invite_type', \IPS\Settings::i()->dp41_rsystem_invite_type, TRUE, array( 'options' => $inviteOpts, 'parse' => 'normal', 'multiple' => false ), NULL, NULL, NULL, 'dp41_rsystem_invite_type' ) );
        
        $form->addTab( 'rsystem_invites_settings' );
        $form->add( new \IPS\Helpers\Form\YesNo( 'dp41_rsystem_enable_invites', \IPS\Settings::i()->dp41_rsystem_enable_invites, FALSE, array( 'togglesOn' => array( 'dp41_rsystem_invites_custom_message', 'dp41_rsystem_invites_the_same_ip', 'dp41_rsystem_invites_resend_attempts', 'dp41_rsystem_invites_invites_per_session', 'dp41_rsystem_invites_day_limit', 'dp41_rsystem_invites_expire_time' ) ), NULL, NULL, NULL, 'dp41_rsystem_enable_invites' ) );
        $form->add( new \IPS\Helpers\Form\YesNo( 'dp41_rsystem_invites_custom_message', \IPS\Settings::i()->dp41_rsystem_invites_custom_message, FALSE, array(), NULL, NULL, NULL, 'dp41_rsystem_invites_custom_message' ) );
        $form->add( new \IPS\Helpers\Form\YesNo( 'dp41_rsystem_invites_the_same_ip', \IPS\Settings::i()->dp41_rsystem_invites_the_same_ip, FALSE, array(), NULL, NULL, NULL, 'dp41_rsystem_invites_the_same_ip' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'dp41_rsystem_invites_resend_attempts', \IPS\Settings::i()->dp41_rsystem_invites_resend_attempts, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'rsystem_disabled_option' ), NULL, NULL, NULL, 'dp41_rsystem_invites_resend_attempts' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'dp41_rsystem_invites_invites_per_session', \IPS\Settings::i()->dp41_rsystem_invites_invites_per_session, FALSE, array( 'min' => 1 ), NULL, NULL, NULL, 'dp41_rsystem_invites_invites_per_session' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'dp41_rsystem_invites_day_limit', \IPS\Settings::i()->dp41_rsystem_invites_day_limit, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'rsystem_disabled_option' ), NULL, NULL, NULL, 'dp41_rsystem_invites_day_limit' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'dp41_rsystem_invites_expire_time', \IPS\Settings::i()->dp41_rsystem_invites_expire_time, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'rsystem_disabled_option' ), NULL, NULL, NULL, 'dp41_rsystem_invites_expire_time' ) );
        
        $displayOpts[ 'settings' ]    = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referral_link_show_mode_settings' );
        $displayOpts[ 'main' ] = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_referral_link_show_mode_main' );
        
        $form->add( new \IPS\Helpers\Form\Select( 'rsystem_referral_link_show_mode', \IPS\Settings::i()->rsystem_referral_link_show_mode, FALSE, array( 'options' => $displayOpts, 'parse' => 'normal', 'multiple' => false ), NULL, NULL, NULL, 'rsystem_referral_link_show_mode' ) );
                
        $form->addTab( 'rsystem_points_integration_settings' );
        $form->add( new \IPS\Helpers\Form\Number( 'rsystem_points_integration_points', \IPS\Settings::i()->rsystem_points_integration_points, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'rsystem_disabled_option' ), NULL, NULL, NULL, 'rsystem_points_integration_points' ) );
        $form->add( new \IPS\Helpers\Form\Text( 'rsystem_points_integration_table', \IPS\Settings::i()->rsystem_points_integration_table, FALSE ) );
        $form->add( new \IPS\Helpers\Form\Text( 'rsystem_points_integration_column', \IPS\Settings::i()->rsystem_points_integration_column, FALSE ) );
        $form->add( new \IPS\Helpers\Form\Number( 'rsystem_points_integration_points_required', \IPS\Settings::i()->rsystem_points_integration_points_required, FALSE, array(), NULL, NULL, NULL, 'rsystem_points_integration_points_required' ) );
        
        $form->addTab( 'rsystem_promotion_settings' );
        $form->add( new \IPS\Helpers\Form\YesNo( 'rsystem_promotion_settings_enable', \IPS\Settings::i()->rsystem_promotion_settings_enable, FALSE, array( 'togglesOn' => array( 'rsystem_promotion_count', 'rsystem_promotion_group', 'rsystem_promotion_protected_groups' ) ), NULL, NULL, NULL, 'rsystem_promotion_settings_enable' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'rsystem_promotion_count', \IPS\Settings::i()->rsystem_promotion_count, FALSE, array(), NULL, NULL, NULL, 'rsystem_promotion_count' ) );
        $form->add( new \IPS\Helpers\Form\Select( 'rsystem_promotion_group', \IPS\Settings::i()->rsystem_promotion_group, FALSE, array( 'options' => $groups ), NULL, NULL, NULL, 'rsystem_promotion_group' ) );
        $form->add( new \IPS\Helpers\Form\Select( 'rsystem_promotion_protected_groups', \IPS\Settings::i()->rsystem_promotion_protected_groups ? explode( ',', \IPS\Settings::i()->rsystem_promotion_protected_groups ) : NULL, FALSE, array( 'options' => $groups, 'parse' => 'normal', 'multiple' => true ), NULL, NULL, NULL, 'rsystem_promotion_protected_groups' ) );
        
		if ( $values = $form->values() )
		{
            /* Multiselect */
            if( $values['dp41_rsystem_groups'] != 'all' )
            {
                $values['dp41_rsystem_groups'] = implode( ",", $values['dp41_rsystem_groups'] );
            }                                        
            else
            {
                $values['dp41_rsystem_groups'] = 'all';
            }
            
            if( $values['dp41_rsystem_redirect_to'] == 'custom' )
            {
                if( empty( $values['dp41_rsystem_redirect_to_custom_url_url'] ) )
                {
                    $values['dp41_rsystem_redirect_to'] == 'index';
                }
            }
            
            $form->saveAsSettings( $values );
            
            \IPS\Session::i()->log( 'acplogs__rsystem_settings_saved' );
		}
        
        \IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'rsystem_title_settings' );
		\IPS\Output::i()->output = $form;    
	}
}