<?php

namespace IPS\rsystem;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Form helper class for linked screenshots
 */
class _Settings
{
    public static $column;
    public static $table;
    public static $memColumnId;
    
    public static $enableNexusIntegration = false;
    
	public function eloInit()
    {
    }
    
    public static function nexusIntegrationIsEnabled()
    {
        return self::$enableNexusIntegration;
    }
    
    public static function checkAccess()
	{
        /* Mod is enabled? */
        if( ! \IPS\Settings::i()->dp41_rsystem_enable )
        {
            return false;
        }

        /* Allowed group?*/
        if( ( \IPS\Settings::i()->dp41_rsystem_groups != 'all' ) && ! \IPS\Member::loggedIn()->inGroup( explode( ',', \IPS\Settings::i()->dp41_rsystem_groups ) ) )
        {
            return false;
        }
        
		/* Banned member? */
		if( \IPS\Member::loggedIn()->dp3_rs_banned )
		{
			return false;
		}
        
        /* Play with hashcode */        
        if( ! \IPS\Member::loggedIn()->dp3_rs_hashcode )
        {
            \IPS\Member::loggedIn()->dp3_rs_hashcode = self::getHashCode( \IPS\Member::loggedIn()->email );
            \IPS\Member::loggedIn()->save();
        }
        
        return true;
	}
    
    public static function getHashCode( $email = '' )
    {
        return md5( $_SERVER['REMOTE_ADDR'] . $email . uniqid( microtime(), true ) );
    }
    
	public static function addPoints( $mid = 0, $minus = false, $_points = 0 )
	{
		/* System is enabled? */
		if( \IPS\Settings::i()->rsystem_points_integration_points == '-1' )
		{
			return false;
		}
        
		/* Set points*/		
		$points	= $_points ? $_points : \IPS\Settings::i()->rsystem_points_integration_points;

		/* Load member info */
		$member = \IPS\Member::load( $mid );
		
		/* Member is banned or referrals are disabled on his account? */
		if( $member->dp3_rs_banned || ! $member->dp3_rs_menable )
		{
			return true;
		}

		/* Check system */
		if( self::_checkSql( 'standard' ) )
		{
			/* Default table is 'members' */
			if( self::$table == 'core_members' )
			{
				/* Set current points */
				$fieldName = self::$column;
                $mPoints = $member->$fieldName;				
				
				/* Plus/minus */
				if( ! $minus )
				{
					$mPoints += $points;
				}
				else
				{
					$mPoints -= $points;
				}
				
				/* Save new points count */
				$member->$fieldName = $mPoints;
                $member->save();
			}
		}
		else
		{
			/* Do we have all required informations? */			
			if( self::_checkSql( 'custom' ) )
			{
				$act = \IPS\Db::i()->select( ' ' . self::$column, self::$table, array( self::$memColumnId . ' = ?', $mid ) )->first();

                /* Set current points status */
				$fieldName = self::$column;
                $mPoints = $act[ $fieldName ];
			
				/* Plus/minus */
				if( ! $minus )
				{
					$mPoints += $points;
				}
				else
				{
					$mPoints -= $points;
				}
				
				/* Save new points count */
                \IPS\Db::i()->update( self::$table, array( self::$column => $mPoints ), array( self::$memColumnId . ' = ?', $mid ) ); 			
			}
		}
		
		/* Return */
		return true;		
	} 
    
	/**
	 * Check table and field exists
	 *
	 * @access	private
	 * @return	bool
	 */		
	public static function _checkSql( $type = '' )
	{
        /* System is enabled? */
		if( \IPS\Settings::i()->rsystem_points_integration_points == '-1' )
		{
			return false;
		}

		/* Set type */
		switch( $type )
		{
			case 'standard':
					self::$table	= \IPS\Settings::i()->rsystem_points_integration_table;
					self::$column	= \IPS\Settings::i()->rsystem_points_integration_column;
			break;	
			
			case 'custom':
				$arr                = explode('|', \IPS\Settings::i()->rsystem_points_integration_table );
                
                if( count( $arr ) > 1 )
                {
                    list( self::$table, self::$memColumnId ) = $arr;
    				
    				self::$column		= \IPS\Settings::i()->rsystem_points_integration_column;
    				
    				/* Check column */
    				if( ! \IPS\Db::i()->checkForColumn( self::$table, self::$memColumnId ) )
    				{
    					return false;
    				}				
				}		
			break;		
		}
	
		/* Check table */
		if( ! \IPS\Db::i()->checkForTable( self::$table ) )
		{		
			return false;
		}

		/* Check column */
		if( ! \IPS\Db::i()->checkForColumn( self::$table, self::$column ) )
		{
			return false;
		}
		
		/* Return */
		return true;
	}
    
	public static function handlePointsSystem( $mid = 0 )
	{
		/* Run promotion checking */
		self::_runPromoteSystem( $mid );
		
		/* System is enabled? */
		if( \IPS\Settings::i()->rsystem_points_integration_points == '-1' )
		{
			return false;
		}
		
		/* No member ID? */
		if( ! $mid )
		{
			return false;
		}
		
		/* Do we have enabled required posts to add points? */		
		if( intval( \IPS\Settings::i()->dp41_rsystem_minimum_posts ) )
		{
			/* Load member */			
			$member = \IPS\Member::load( $mid );
			
			if( $member->member_posts < \IPS\Settings::i()->dp41_rsystem_minimum_posts )
			{
				return false;
			}
		}
		
		/* Still here? Add them! */
		self::addPoints( $mid );
		
		/* Return to clarify */
		return true;
	}
    
	protected static function _runPromoteSystem( $mid = 0 )
	{
		/* Run sending notification system */
		self::_runNotificationSystem( $mid );
		
		/* INIT */
		$promoteCount = intval( \IPS\Settings::i()->rsystem_promotion_count );
		
		$promoteGroup = intval( \IPS\Settings::i()->rsystem_promotion_group );		
		
		/* System is enabled? */
		if( ! \IPS\Settings::i()->rsystem_promotion_settings_enable )
		{
			return;
		}
		
		/* Zero referrers? Return too! */		
		if( ! $promoteCount )
		{
			return;
		}

		/* Check promote group */
		if( ! $promoteGroup )
		{
			return;
		}
				
		/* No member ID? */
		if( ! $mid )
		{
			return;
		}
			
		/* Load member informations */
		$member = \IPS\Member::load( $mid );
		
		/* Members is in protected group? */
		if( in_array( $member->member_group_id, explode(',', \IPS\Settings::i()->rsystem_promotion_protected_groups ) ) )
		{
			return;
		}
		
		/* Get actual referred users by this member */
		$cnt = \IPS\Db::i()->select( 'COUNT(*)', 'core_members', array( "dp3_rs_referred_by = ?", $member->member_id ) )->first();
		
        /* Check promotion */
		if( $cnt >= $promoteCount )
		{
            $member->member_group_id = $promoteGroup;
		}
		
		/* Return to clarify */
		return true;
	}
    
	public static function _runNotificationSystem( $mid = 0 )
	{
        $title = \IPS\Member::loggedIn()->language()->get( 'rsystem_not_title' );
        $msg   = \IPS\Member::loggedIn()->language()->get( 'rsystem_not_content' );
    
		/* Title and message are set? */
		if( trim( $title ) && trim( $msg ) )
		{
			/* Load member info */			
			$user = \IPS\Member::load( $mid );
					
    		/* Build search and replace arrays */
    		$search 	= array(     '/<#member_name#>/',
                                     '/<#board_name#>/',
                                     '/<#board_url#>/'
            );
    		
    		$replace	= array( 
    						$user->name,
                            \IPS\Settings::i()->board_name,
                            \IPS\Settings::i()->board_url
    		);
            $title = preg_replace( $search, $replace, $title );
            $msg   = preg_replace( $search, $replace, html_entity_decode( $msg ) );	
             
            /* Send it */           
			$email			= \IPS\Email::buildFromContent( $title, $msg, $msg );
            $email->from	= \IPS\Settings::i()->email_out;
			$email->send( $user->email );
		}
		
		/* Sort it out and return */
		return;				
	}
    
	public static function sendMail( $data, $resend = false )
	{	        
        /* Do we have title and content of mail? */
        $title = \IPS\Member::loggedIn()->language()->get( 'rsystem_invite_title' ); 
		
        /* Build search and replace arrays */	
		$search 	= array(     '/<#user_msg#>/',
                                 '/<#aff_link#>/',
                                 '/<#board_name#>/',
                                 '/<#board_url#>/'
         );
		
		$replace	= array( 
						$data->body,
                        \IPS\Http\Url::internal( 'app=rsystem&key=' . $data->secure_key, 'front', '' ),
                        \IPS\Settings::i()->board_name,
                        \IPS\Settings::i()->board_url
		);

        $msg   = preg_replace( $search, $replace, html_entity_decode( \IPS\Member::loggedIn()->language()->get( 'rsystem_invite_content' ) ) );	
         
        /* Send it */
		$email			= \IPS\Email::buildFromContent( $title, $msg, $msg );
        $email->from	= \IPS\Settings::i()->email_out;
		$email->send( $data->friend_mail );	
	}    

    public static function whereToRedirect()
    {
        if( \IPS\Settings::i()->dp41_rsystem_redirect_to == 'index' )
        {
            return \IPS\Http\Url::internal( 'app=forums&module=forums&controller=index', 'front', 'forums' );
        }
        
        if( \IPS\Settings::i()->dp41_rsystem_redirect_to == 'custom' )
        {
            if ( filter_var( \IPS\Settings::i()->dp41_rsystem_redirect_to_custom_url_url, FILTER_VALIDATE_URL ) !== false )
            {
                return \IPS\Http\Url::external( \IPS\Settings::i()->dp41_rsystem_redirect_to_custom_url_url );
            }
        }
        
        return \IPS\Http\Url::internal( 'app=core&module=system&controller=register', 'front', 'register', NULL, \IPS\Settings::i()->logins_over_https );
    }
    
	public static function playWithCheckKey()
	{
		/* INIT */
		$myIP = $_SERVER['REMOTE_ADDR'];
			
		/* Get cookies */
		if( isset( \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'] ) )
        {
            $reff_cookie = \IPS\Request::i()->cookie['dp3_rs_invite_reff_hash'];
		}
        else
        {
            $reff_cookie = '-';
        }
        
		/* Do we have any cookie? */
		if( ( $reff_cookie != '-' ) && \IPS\Member::loggedIn()->member_id )
		{
			/* User can be referred only once */
			try
            {               
                $checkMember = \IPS\Db::i()->select( 'COUNT(*) as count', 'rsystem_referrals', array( 'i_status = ? AND i_invited_id = ?', 'used', \IPS\Member::loggedIn()->member_id ) )->first();

                if( $checkMember )
                {
    				/* Delete cookie */
                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
                    \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
                    return;	
                }
            }
            catch ( \UnderflowException $e )
            {}
            
            /* Get informations from SQL */
			try
            {
                $data = \IPS\Db::i()->select( '*', 'rsystem_referrals', array( 'i_secure_key = ?', $reff_cookie ) )->first();			
			}
            catch( \UnderflowException $e )
            {
                return;
            }
        
			if( ! $data['i_id'] || ( $data['i_status'] == 'used' ) || ( $data['i_inviter_id'] == \IPS\Member::loggedIn()->member_id ) || ! self::checkActIp() ) 
			{
				/* Delete cookie */
                \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
                \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
                return;			
			}
			else
			{
				/* Check day limit */				
				$dtime	= time() - ( 60 * 60 * 24 );
				
				$count  = \IPS\Db::i()->select( 'COUNT(*) as count', 'rsystem_referrals', array( 'i_time >= ? AND i_inviter_id = ?', $dtime, $data['i_inviter_id'] ) )->first();
				
				/* Check day limit */				
				if( \IPS\Settings::i()->dp41_rsystem_invites_day_limit <= $count ) 
				{
					return;			
				}															
				
				/* Add points */
				$success = self::handlePointsSystem( $data['i_inviter_id'] );
		
				/* Update SQL key */
			    \IPS\Db::i()->update( 'rsystem_referrals', array( 'i_status' => 'used', 'i_invited_ip' => $myIP, 'i_invited_id' => \IPS\Member::loggedIn()->member_id ), array( array( "i_secure_key = ?", $reff_cookie ) ) );
                
                /* Add referral */
                \IPS\Db::i()->update( 'core_members', array( 'dp3_rs_referred_by' => $data['i_inviter_id'] ), array( 'member_id = ?', \IPS\Member::loggedIn()->member_id ) );    
                
                /* Try add nexus info */
                if( \IPS\rsystem\Settings::nexusIntegrationIsEnabled() )
                {
        			$referredBy = \IPS\Member::load( $data['i_inviter_id'] );
        			if ( $referredBy->member_id )
        			{  
                        try
                		{
                			$existsReferreral = \IPS\Db::i()->select( '*', 'nexus_referrals', array( 'member_id=? AND referred_by=?', \IPS\Member::loggedIn()->member_id, $referredBy->member_id ) )->first();
                		}
                		catch ( \UnderflowException $e )
                		{
                            \IPS\Db::i()->insert( 'nexus_referrals', array(
                                					'member_id'		=> \IPS\Member::loggedIn()->member_id,
                                					'referred_by'	=> $referredBy->member_id,
                                					'amount'		=> '',
                            ) );
                		}
                        
        			}
                }
                
				/* Delete cookie */
                \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_id', 0 );
                \IPS\Request::i()->setCookie( 'dp3_rs_invite_reff_hash', 0 );
                return;					
			}
		}		
	}
    
	public static function checkActIp()
	{
		/* INIT */
		$myIP = $_SERVER['REMOTE_ADDR'];
		
		/* Allow the same IP's? */
		if( \IPS\Settings::i()->dp41_rsystem_invites_the_same_ip )
		{
			return true;
		}
		
        $remote = false;
        
		/* Get remote ip data */
		if( $myIP )
		{
			try
            {
                $remote	= \IPS\Db::i()->select( 'i_id', 'rsystem_referrals', array( 'i_invited_ip = ?', 'used', $myIP ) )->first();
            }
            catch ( \UnderflowException $e )
            {}
		}
		
		if( $remote )
		{
			return false;
		} 
		
		return true;			
	}
    
	public static function checkEnteredName( $name = '' )
	{
		/* No name? */
		if( ! mb_strlen( $name ) )
		{
			return false;
		}
		
		/* Load member */
		$member = \IPS\Member::load( $name, 'name' );
		
		/* Check allowed group */
		if( \IPS\Settings::i()->dp41_rsystem_groups != 'all' )
        {
            if( ! in_array( $member->member_group_id, explode( ',', \IPS\Settings::i()->dp41_rsystem_groups ) ) )
    		{
    			return 'rsystem_dp3_rs_error_not_allowed_group';
    		}
        }
		
		/* Check referred enabled */
		if( ! $member->dp3_rs_menable )
		{
			return 'rsystem_dp3_rs_error_not_enabled';
		}
		
		/* Still here? OK! */
		return $member->member_id;
	}
	
}